unit F_Convert;

interface

{$IFDEF FPC}
{$Mode Delphi}
{$R *.lfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}

uses
{$IFDEF FPC}
  EditBtn,
{$ELSE}
  Mask, rxToolEdit,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, FileCtrl,
  ExtCtrls,
  UTF8Process,
  Process,
  SynEdit,
  SynHighlighterPas;

type

  { TForm1 }

  TForm1 = class(TForm)
    DirectoryEdit: TDirectoryEdit;
    Panel1: TPanel;
    FileListBox: TFileListBox;
    Convertir: TButton;
    FilterComboBox: TFilterComboBox;
    SynPasSyn : TSynPasSyn;
    SynEdit : TSynEdit;
    Messages: TMemo;
    Panel2: TPanel;
    procedure ConvertirClick(Sender: TObject);
    procedure DirectoryEditChange(Sender: TObject);
    procedure FilterComboBoxChange(Sender: TObject);
    procedure MessagesChange(Sender: TObject);
    procedure MessagesClick(Sender: TObject);
  private
    as_file : String;
    sr_Ranges : TSynPasSynRange;
    constructor Create ( AOwner : TComponent ); override;
    procedure ConvertirDir(const as_Directory: String);
    procedure ConvertitField(var as_Ligne: String; const ai_LengthBegin: Integer
      );
    procedure DeleteUntilEnd(const ast_Lignes: TStringList ; const ai_pos1, ai_posEnd : LongInt );
    { Dclarations prives }
  public
    procedure ConvertitLignePAS( var ast_Lignes : TStringList; as_Lignes : String ); virtual;
    procedure ConvertitFichier ( const FilePath, FileName  : String );
    { Dclarations publiques }
  end;
  TSynPasHack = class(TSynPasSyn)
    public
      property CurrentRanges;
  end;

var
  Form1: TForm1;

implementation

uses StrUtils, LazUTF8,LazFileUtils,LConvEncoding;

var gb_Posifndef_fpc : Boolean = False ;
    gb_report        : Boolean = False ;

    // Recursive procedure to build a list of files
function  fb_FindFiles( const astl_FilesList: TStrings; as_StartDir : String;
                        const ab_ListFiles : Boolean = True ; const ab_ListDirs : Boolean = True;
                        const ab_FullPath : Boolean = False;
                        const as_FileMask: String='*'; const as_DirMask: String='*'):Boolean;
var
  SR: TSearchRec;
  IsFound: Boolean;
  ls_Path : String;
begin
  Result := False ;

  if ab_FullPath
   Then ls_Path := as_StartDir
   Else ls_Path := '' ;

  { Build a list of the files in directory as_StartDir
     (not the directories!)                         }
  if ab_ListDirs Then
  try
    IsFound := FindFirstUTF8(as_StartDir + as_DirMask, faDirectory, SR) = 0 ;
    while IsFound do
     begin
      with SR do
      if (( Name > '' ) and ( Name <> '.' ) and ( Name <> DirectorySeparator ) and ( Name <> '..' ))
      and DirectoryExistsUTF8 ( as_StartDir + Name )
       then astl_FilesList.Add(ls_Path + Name);
      IsFound := FindNextUTF8(SR) = 0;
      Result := True ;
    end;
    FindCloseUTF8(SR);
  Except
    FindCloseUTF8(SR);
  End ;
  if ab_ListFiles Then
  try
    IsFound := FindFirstUTF8(as_StartDir+as_FileMask, faAnyFile-faDirectory, SR) = 0;
    while IsFound do
     begin
        with SR do
        if (( Name > '' ) and ( Name <> '.' ) and ( Name <> DirectorySeparator ) and ( Name <> '..' ))
         and  FileExistsUTF8 ( as_StartDir + Name )
         Then
          astl_FilesList.Add(ls_Path + Name);
        IsFound := FindNextUTF8(SR) = 0;
        Result := True ;
      end;
    FindCloseUTF8(SR);
  Except
    FindCloseUTF8(SR);
  End ;

end;


procedure TForm1.ConvertirClick(Sender: TObject);
var ls_Directory : String;
begin
  Messages.Lines.Clear;
  ls_Directory := DirectoryEdit.{$IFDEF FPC}Directory{$ELSE}Text{$ENDIF};
  ConvertirDir ( DirectoryEdit.{$IFDEF FPC}Directory{$ELSE}Text{$ENDIF} );
  Messages.Lines.Add( '-> Tout est fini .' );
end;

procedure TForm1.DirectoryEditChange(Sender: TObject);
begin
  FileListBox.Directory := DirectoryEdit.{$IFDEF FPC}Directory{$ELSE}Text{$ENDIF} ;
end;

procedure TForm1.FilterComboBoxChange(Sender: TObject);
begin
  FileListBox.Mask := FilterComboBox.Mask;
end;

procedure TForm1.MessagesChange(Sender: TObject);
begin

end;

procedure TForm1.MessagesClick(Sender: TObject);
var ls_Temp : String;
    Process:TProcessUTF8;
begin
  with Messages do
   Begin
      if Lines.Count < 2 Then Exit;
      ls_Temp := Lines[CaretPos.y];
      if pos('(',ls_Temp) = 0
       Then
        if CaretPos.y = 0
         Then ls_Temp := Lines[1]
         Else ls_Temp := Lines[CaretPos.y-1];
      ls_Temp:= IncludeTrailingPathDelimiter(DirectoryEdit.Text)+ copy(ls_Temp,1,pos('(',ls_Temp)-1);
      if FileExistsUTF8(ls_Temp)
       then
        Begin
         Process := TProcessUTF8.Create(nil);
         with Process do
           try
             Options := Options + [poWaitOnExit]; // wait for result
             // parameters property not read on linux and windows
             CommandLine:='xdg-open'+' '+ls_Temp;
             Execute; // will the command work ?
           finally
             Destroy;
           end;
         End;
   end;
end;


constructor TForm1.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SynPasSyn := TSynPasSyn.Create(Self);

  SynEdit := TSynEdit.Create(Self);

  SynEdit.Highlighter:=SynPasSyn;
  DirectoryEdit.Text:=ExtractFileDir(Application.ExeName);
end;

procedure TForm1.ConvertirDir(const as_Directory : String );
var li_i : Integer ;
    lstl_StringList : TStringList;
begin
  lstl_StringList := TStringList.Create;
  if fb_FindFiles ( lstl_StringList, IncludeTrailingPathDelimiter(as_Directory), True, True, True ) Then
    Begin
      for li_i := 0 to lstl_StringList.Count - 1 do
        if (DirectoryExistsUTF8 ( lstl_StringList [ li_i ] )) then
          Begin
            ConvertirDir(lstl_StringList [ li_i ] );
          End
         else
           if (FileExistsUTF8 ( lstl_StringList [ li_i ] )) then
            Begin
              ConvertitFichier ( lstl_StringList [ li_i ], ExtractFileName ( lstl_StringList [ li_i] ));
            End;

    End;
  lstl_StringList.free;
end;

procedure TForm1.ConvertitField( var as_Ligne : String ; const ai_LengthBegin : Integer );
var li_i, li_2Maj : Integer ;
    lb_Maj, lb_Maj2 : Boolean ;
begin
  lb_Maj  := False ;
  lb_Maj2 := False ;
  li_2Maj := 0;
  for li_i := ai_LengthBegin to length ( as_Ligne ) do
    Begin
      if ( as_Ligne [ li_i ] = '.' ) Then
        Begin
          if lb_Maj2 Then
            as_Ligne := copy(as_Ligne, 1, li_2Maj-1) +
                        '.FieldByName(''' +
                        copy(as_Ligne, li_2Maj, li_i -li_2Maj) +
                        ''')' +
                        copy(as_Ligne, li_i, length ( as_Ligne ) - li_i +1) ;
          Exit;
        end;
      if ( as_Ligne [ li_i ] in ['A'..'Z','_'] ) Then
        Begin
          if lb_Maj Then
            Begin
              if not lb_Maj2 Then
                li_2Maj := li_i - 1;
              lb_Maj2 := True;
            end;
          lb_Maj := True;
        end
       Else
        if as_Ligne [ li_i ] <> ' ' Then
          Begin
            lb_Maj  := False ;
            lb_Maj2 := False ;
          end;
    end;
End;

procedure p_ReplaceButton ( var as_Ligne : String ; const as_replaceEnd : String );
Begin
  as_Ligne := StringReplace ( as_Ligne, 'TJvXPButton', 'TFW'+as_replaceEnd, [rfIgnoreCase] );
  as_Ligne := StringReplace ( as_Ligne, 'TFlatSpeedButton', 'TFS'+as_replaceEnd, [rfIgnoreCase] );
  as_Ligne := StringReplace ( as_Ligne, 'TFlatButton', 'TFB'+as_replaceEnd, [rfIgnoreCase] );
  as_Ligne := StringReplace ( as_Ligne, 'TBlueFlatSpeedButton', 'TBFS'+as_replaceEnd, [rfIgnoreCase] );
end;



procedure TForm1.ConvertitFichier(const FilePath, FileName : String);
var ls_Lignes : WideString ;
    lb_DFM ,
    lb_LFM ,
    lb_lrs ,
    lb_Pas ,
    lb_lpr ,
    lb_dpr : Boolean ;
    LType : Pointer;
    li_i : Integer;
begin
  lb_DFM := PosEx ( '.dfm', LowerCase ( FilePath ), length ( FilePath ) - 5 ) >= length ( FilePath ) - 5 ;
  lb_LFM := PosEx ( '.lfm', LowerCase ( FilePath ), length ( FilePath ) - 5 ) >= length ( FilePath ) - 5 ;
  lb_pas := PosEx ( '.pas', LowerCase ( FilePath ), length ( FilePath ) - 5 ) >= length ( FilePath ) - 5 ;
  lb_lrs := PosEx ( '.lrs', LowerCase ( FilePath ), length ( FilePath ) - 5 ) >= length ( FilePath ) - 5 ;
  lb_lpr := PosEx ( '.lpr', LowerCase ( FilePath ), length ( FilePath ) - 5 ) >= length ( FilePath ) - 5 ;
  lb_dpr := PosEx ( '.dpr', LowerCase ( FilePath ), length ( FilePath ) - 5 ) >= length ( FilePath ) - 5 ;
  if ( lb_Pas or lb_lrs or lb_lpr or lb_dpr ) then
    try
     as_file := Copy(FilePath,Length(DirectoryEdit.Text)+1,Length(FilePath));
     {if ch_utf8.Checked Then
       Begin
         lst_Lignes.LoadFromFile ( FilePath );
         lst_Lignes.text := ISO_8859_1ToUTF8(lst_Lignes.Text);
         lst_Lignes.SaveToFile   ( FilePath );
       end;                                }
      SynEdit.Lines.LoadFromFile ( FilePath );
      SynPasSyn.ScanRanges;
      with SynPasSyn,SynEdit.Lines,Messages do
       for li_i := 0 to Count - 1 do
         Begin
           if (Strings [ li_i ]= 'fofgle') Then
             Begin
              Append(as_file+'('+IntToStr(li_i)+'):'+' '+BoolToStr(rsInTypeBlock in TRangeStates(Integer(PtrUInt(TSynPasSynRange((TSynPasHack (SynPasSyn)).CurrentRanges [ li_i ]).RangeType ))))+' '+IntToStr(FoldTypeAtNodeIndex ( li_i, 0, True ))+' '+BoolToStr(IsLineStartingInDirective ( li_i ))+' '+' '+BoolToStr(FoldBlockNestedTypes ( li_i, 1, LType ))+' '+IntToStr(FoldBlockOpeningCount ( li_i ))+' '+IntToStr(FoldBlockClosingCount ( li_i ))+' '+IntToStr(FoldBlockEndLevel ( li_i ))+' '+IntToStr(FoldBlockMinLevel ( li_i ))+' ');
              Append(Strings [ li_i ]);
             end;
           if (16896=integer( TRangeStates(Integer(PtrUInt(TSynPasSynRange((TSynPasHack (SynPasSyn)).CurrentRanges [ li_i ]).RangeType ))))) and (FoldTypeAtNodeIndex ( li_i, 0, True )=1 ) and not IsLineStartingInDirective ( li_i ) and not FoldBlockNestedTypes ( li_i, 0, LType ) and ( FoldBlockEndLevel ( li_i ) = 1 ) and ( FoldBlockOpeningCount ( li_i ) = 0 ) and ( FoldBlockClosingCount ( li_i ) = 0 ) Then
             Begin
              Append(as_file+'('+IntToStr(li_i)+'):'+' '+BoolToStr(rsInTypeBlock in TRangeStates(Integer(PtrUInt(TSynPasSynRange((TSynPasHack (SynPasSyn)).CurrentRanges [ li_i ]).RangeType ))))+' '+IntToStr(FoldTypeAtNodeIndex ( li_i, 0, True ))+' '+BoolToStr(IsLineStartingInDirective ( li_i ))+' '+' '+BoolToStr(FoldBlockNestedTypes ( li_i, 1, LType ))+' '+IntToStr(FoldBlockOpeningCount ( li_i ))+' '+IntToStr(FoldBlockClosingCount ( li_i ))+' '+IntToStr(FoldBlockEndLevel ( li_i ))+' '+IntToStr(FoldBlockMinLevel ( li_i ))+' ');
              Append(Strings [ li_i ]);
             end;
{          if FoldBlockEndLevel(li_i) = 0 Then
            Begin
             Append(as_file+'('+IntToStr(li_i)+'):'+IntToStr(FoldBlockEndLevel(li_i)));
             Append(Strings [ li_i ]);
            end;}
         end;
    Application.ProcessMessages;
  finally
  end;

end;

function fb_IsField ( const as_ligne : String ) : Boolean;
Begin
  Result :=
       ( pos ( 'TIBStringField', as_Ligne )> 0 )
    or ( pos ( 'TIBBlobField', as_Ligne )> 0 )
    or ( pos ( 'TIBIntegerField', as_Ligne )> 0 )
    or ( pos ( 'TIBDateTimeField', as_Ligne )> 0 )
    or ( pos ( 'TIBFloatField', as_Ligne )> 0 )
    or ( pos ( 'TBlobField', as_Ligne )> 0 )
    or ( pos ( 'TFMTBCDField', as_Ligne )> 0 )
    or ( pos ( 'TDateTimeField', as_Ligne )> 0 )
    or ( pos ( 'TTimeField', as_Ligne )> 0 )
    or ( pos ( 'TDateField', as_Ligne )> 0 )
    or ( pos ( ':TIdHTTP', as_Ligne )> 0 )
    or ( pos ( ': TIdAntiFreeze', as_Ligne )> 0 )
    or ( pos ( ':TIdEncoderMIME', as_Ligne )> 0 )
    or ( pos ( 'TIntegerField', as_Ligne )> 0 )
    or ( pos ( 'TStringField', as_Ligne )> 0 )
    or ( pos ( 'TFloatField', as_Ligne )> 0 )
    or ( pos ( 'TMemoField', as_Ligne )> 0 );
End;


function fb_posUnwanted ( const as_Begin, as_ligne : String ; const ab_Grid : Boolean ) : Boolean ;
Begin
  Result := ( pos ( as_Begin + 'TExtDBGridBandedTableViewStyleSheet', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TExtDBGridTableViewStyleSheet', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TDBCheckVirtualDBTreeExReportLink', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TDBCheckVirtualDBTreeExStyleSheet', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TdxCustomContainerReportLink', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TdxCustomContainerReportLink', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TExtraOptions', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TRLDetailGrid', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TcxIntlPrintSys3', as_Ligne )> 0 )
         or ( pos ( as_Begin + 'TcxIntlBars', as_Ligne )> 0 )
         or ( pos ( as_Begin + 'TcxIntl', as_Ligne )> 0 )
         or ( pos ( as_Begin + 'TDBCheckVirtualDBTreeExColumn', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TExtDBGridLevel', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TcxEditStyleController', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TWizardTree', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TcxHintStyleController', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TdxPrintStyleManager', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TBasedxPrintStyle', as_ligne )> 0 )

         or ( pos ( as_Begin + 'TcxStyle', as_ligne )> 0 );
  if not Result
  and ab_Grid Then
  Result := ( pos ( as_Begin + 'TExtDBGridDBColumn', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TExtDBGridDBBandedTableView', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TExtDBGridDBTableView', as_ligne )> 0 )
         or ( pos ( as_Begin + 'TExtDBGridDBBandedColumn ', as_ligne )> 0 );

end;


procedure TForm1.ConvertitLignePAS( var ast_Lignes : TStringList; as_Lignes : String );
var li_pos1, li_pos2 : LongInt;
    ls_Ligne : String;

begin
  li_pos1 := -1;
  ast_Lignes.BeginUpdate;
  while li_pos1 < ast_Lignes.Count - 1 do
    Begin
      inc ( li_pos1 );
      Application.ProcessMessages;
      ls_Ligne := ast_Lignes [ li_pos1 ];
      li_pos2 := pos ( '.FieldByName(''', ls_Ligne );
      while ( li_pos2 > 0 ) do
        Begin
          ls_Ligne:=copy(ls_Ligne, 1, li_pos2 -1 )+copy ( ls_Ligne, li_pos2 + 14, length ( ls_Ligne ) - li_pos2 - 13 );
          li_pos2 := PosEx(''')',ls_Ligne,li_pos2);
          if li_pos2 > 0 Then ls_Ligne := copy(ls_Ligne, 1, li_pos2 -1 )+copy ( ls_Ligne, li_pos2 + 2, length ( ls_Ligne ) - li_pos2 - 1 );
          li_pos2 := pos ( '.FieldByName(''', ls_Ligne );
         // ShowMessage(ls_Ligne)
        end;
      ls_Ligne := StringReplace ( ls_Ligne, 'RecordCount=0', 'IsEmpty', [rfIgnoreCase,rfReplaceAll] );
      ls_Ligne := StringReplace ( ls_Ligne, 'RecordCount= 0', 'IsEmpty', [rfIgnoreCase,rfReplaceAll] );
      ls_Ligne := StringReplace ( ls_Ligne, 'RecordCount = 0', 'IsEmpty', [rfIgnoreCase,rfReplaceAll] );
      ls_Ligne := StringReplace ( ls_Ligne, 'RecordCount <> 0', 'not IsEmpty', [rfIgnoreCase,rfReplaceAll] );
      ls_Ligne := StringReplace ( ls_Ligne, 'RecordCount<> 0', 'not IsEmpty', [rfIgnoreCase,rfReplaceAll] );
      ls_Ligne := StringReplace ( ls_Ligne, 'RecordCount<>0', 'not IsEmpty', [rfIgnoreCase,rfReplaceAll] );
      ls_Ligne := StringReplace ( ls_Ligne, 'RecordCount>0', 'not IsEmpty', [rfIgnoreCase,rfReplaceAll] );
      ls_Ligne := StringReplace ( ls_Ligne, 'RecordCount > 0', 'not IsEmpty', [rfIgnoreCase,rfReplaceAll] );
      ast_Lignes [ li_pos1 ] := ls_Ligne;

    End;
end;


procedure TForm1.DeleteUntilEnd( const ast_Lignes : TStringList ; const ai_pos1, ai_posEnd : LongInt );
var ls_Ligne : String;
Begin
  repeat
  ls_Ligne := ast_Lignes [ ai_pos1 ];
  ast_Lignes.Delete( ai_pos1);
  until ( pos ( 'end', ls_Ligne ) > 0 )
    and (( ai_posEnd = -1 ) or ( ai_posEnd = pos ( 'end', ls_Ligne )));
end;

end.
